# -*- coding: iso-8859-15 -*-

##############################################################################
#
# Copyright (c) 2006 Ingeniweb SAS
#
# This software is subject to the provisions of the GNU General Public License,
# Version 2.0 (GPL).  A copy of the GPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Testing rotatezlogs

$Id: test_rotatezlogs.py,v 1.2 2006/05/02 09:17:07 glenfant Exp $
"""

import unittest
import tempfile
import logging
import os

from ZConfig.components.logger.tests import test_logger
from ZConfig import ConfigurationSyntaxError

from Products.rotatezlogs.LogHandlers import RotateFileHandler


class TestRotatelogConfig(test_logger.TestConfig):
    """Note that we run all original tests too for regression purpose
    to check if our config doesn't screw up standard config.  See base
    class to see how we create new tests """

    _schematext = """
      <schema>
        <import package="ZConfig.components.logger"/>
        <import package="Products.rotatezlogs"/>
        <section type="eventlog" name="*" attribute="eventlog"/>
      </schema>
      """

    def test_with_rotatelog1(self):
        """With rotatezlog, no path"""

        logger_def = (
            "<eventlog>\n"
            "  <rotatelogfile>\n"
            "  </rotatelogfile>\n"
            "</eventlog>")
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)


    def test_with_rotateziplog1(self):
        """With zip rotatezlog, no path"""

        logger_def = (
            "<eventlog>\n"
            "  <rotateziplogfile>\n"
            "  </rotateziplogfile>\n"
            "</eventlog>")
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)
        return


    def test_with_rotatelog2(self):
        """With rotatezlog with path but no max-bytes"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotatelogfile>\n"
            "    path %s\n"
            "  </rotatelogfile>\n"
            "</eventlog>" % fn)
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)
        return


    def test_with_rotateziplog2(self):
        """With zip rotatezlog with path but no max-bytes"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotateziplogfile>\n"
            "    path %s\n"
            "  </rotateziplogfile>\n"
            "</eventlog>" % fn)
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)
        return


    def test_with_rotatelog3(self):
        """With rotatezlog with path, max-bytes, no backup-count"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotatelogfile>\n"
            "    path %s\n"
            "    max-bytes 100KB\n"
            "  </rotatelogfile>\n"
            "</eventlog>" % fn)
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)
        return


    def test_with_rotateziplog3(self):
        """With zip rotatezlog with path, max-bytes, no backup-count"""
        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotateziplogfile>\n"
            "    path %s\n"
            "    max-bytes 100KB\n"
            "  </rotateziplogfile>\n"
            "</eventlog>" % fn)
        self.failUnlessRaises(ConfigurationSyntaxError,
                              self.check_simple_logger,logger_def)
        return


    def test_with_rotatelog4(self):
        """With rotatezlog with all options set"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotatelogfile>\n"
            "    path %s\n"
            "    max-bytes 100KB\n"
            "    backup-count 5\n"
            "    level warning\n"
            "  </rotatelogfile>\n"
            "</eventlog>" % fn)
        logger = self.check_simple_logger(logger_def)
        handler = logger.handlers[0]
        self.failUnless(isinstance(handler, RotateFileHandler))
        self.assertEqual(handler.maxBytes, 100 * 1024)
        self.assertEqual(handler.backupCount, 5)
        self.assertEqual(handler.level, logging.WARNING)
        handler.close()
        os.remove(fn)
        return

    def test_with_rotateziplog4(self):
        """With zip rotatezlog with all options set"""
    
        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotateziplogfile>\n"
            "    path %s\n"
            "    max-bytes 100KB\n"
            "    backup-count 5\n"
            "    level warning\n"
            "  </rotateziplogfile>\n"
            "</eventlog>" % fn)
        logger = self.check_simple_logger(logger_def)
        handler = logger.handlers[0]
        self.failUnless(isinstance(handler, RotateFileHandler))
        self.assertEqual(handler.maxBytes, 100 * 1024)
        self.assertEqual(handler.backupCount, 5)
        self.assertEqual(handler.level, logging.WARNING)
        handler.close()
        os.remove(fn)
        return


    def test_with_rotatelog5(self):
        """We'll try to fill and rotate some logs"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotatelogfile>\n"
            "    path %s\n"
            "    max-bytes 2KB\n"
            "    backup-count 2\n"
            "    level warning\n"
            "  </rotatelogfile>\n"
            "</eventlog>" % fn)
        logger = self.check_simple_logger(logger_def)
        handler = logger.handlers[0]
        for i in xrange(1000):
            logger.error("Houston, we got a problem")

        # Check we have rotated
        basename = os.path.basename(fn)
        dirname = os.path.dirname(fn)
        for i in xrange(1, handler.backupCount +1):
            ext = '.%d' % i
            rot_log_path = os.path.join(dirname, basename + ext)
            self.failUnless(os.path.exists(rot_log_path))
            os.remove(rot_log_path)
        handler.close()
        os.remove(fn)
        return

    def test_with_rotateziplog5(self):
        """We'll try to fill and rotate and zip some logs"""

        fn = tempfile.mktemp()
        logger_def = (
            "<eventlog>\n"
            "  <rotateziplogfile>\n"
            "    path %s\n"
            "    max-bytes 2KB\n"
            "    backup-count 2\n"
            "    level warning\n"
            "  </rotateziplogfile>\n"
            "</eventlog>" % fn)
        logger = self.check_simple_logger(logger_def)
        handler = logger.handlers[0]
        for i in xrange(1000):
            logger.error("Houston, we got a problem")

        # Check we have rotated as specified
        basename = os.path.basename(fn)
        dirname = os.path.dirname(fn)
        for i in xrange(1, handler.backupCount +1):
            ext = '.%d.zip' % i
            rot_log_path = os.path.join(dirname, basename + ext)
            self.failUnless(os.path.exists(rot_log_path))
            os.remove(rot_log_path)
        handler.close()
        os.remove(fn)
        return

# /class TestRotatelogConfig

def test_suite():
    return unittest.makeSuite(TestRotatelogConfig)

if __name__ == '__main__':
    unittest.main(defaultTest="test_suite")
